#!/usr/bin/env bash
# Copyright 2022 Florian Schmaus, Florian Fischer
set -euo pipefail

if [[ -z "$1" ]] || [[ ! -f "$1" ]]; then
	echo "usage: ${0##*/} PERF-DATA"
	exit 2
fi

PERF=${PERF:-perf}
PERF_DATA="$1"

# Pretty fancy method to get reliable the absolute path of a shell
# script, *even if it is sourced*. Credits go to GreenFox on
# stackoverflow: http://stackoverflow.com/a/12197518/194894
pushd . > /dev/null
SCRIPTDIR="${BASH_SOURCE[0]}";
while [ -h "${SCRIPTDIR}" ]; do
	cd "$(dirname "${SCRIPTDIR}")"
	SCRIPTDIR="$(readlink "$(basename "${SCRIPTDIR}")")";
done
cd "$(dirname "${SCRIPTDIR}")" > /dev/null
SCRIPTDIR="$(pwd)";
popd > /dev/null

DATE=$(date -Iminutes)
TMPDIR=$(mktemp --tmpdir=/var/tmp --directory "flamegraph-${DATE}-XXXXX")
pushd "${TMPDIR}" > /dev/null

${PERF} script -i "${PERF_DATA}" > out.perf
"${SCRIPTDIR}"/stackcollapse-perf.pl out.perf > out.folded
sed -e 's/^iou-wrk-[0-9]\+;//' out.folded > out.folded.filtered
"${SCRIPTDIR}"/flamegraph.pl out.folded.filtered > flame.svg

popd >/dev/null

FLAMEGRAPH="$(dirname "${PERF_DATA}")"/"$(basename "${PERF_DATA}" .data)"-flamegraph.svg
cp "${TMPDIR}/"flame.svg "${FLAMEGRAPH}"
